@@ -26,6 +26,8 @@
 
 #include "config.h"
 
+#include "gtkmain.h"
+
 #include <glib.h>
 #include "gdkconfig.h"
 
@@ -52,7 +54,6 @@
 #include "gtkclipboard.h"
 #include "gtkdnd.h"
 #include "gtkversion.h"
-#include "gtkmain.h"
 #include "gtkmodules.h"
 #include "gtkrc.h"
 #include "gtkrecentmanager.h"
@@ -63,6 +64,8 @@
 #include "gtktooltip.h"
 #include "gtkdebug.h"
 #include "gtkalias.h"
+#include "gtkmenu.h"
+#include "gdk/gdkkeysyms.h"
 
 #include "gdk/gdkprivate.h" /* for GDK_WINDOW_DESTROYED */
 
@@ -384,9 +387,9 @@ static gboolean do_setlocale = TRUE;
  * 
  * Prevents gtk_init(), gtk_init_check(), gtk_init_with_args() and
  * gtk_parse_args() from automatically
- * calling <literal>setlocale (LC_ALL, "")</literal>. You would 
- * want to use this function if you wanted to set the locale for 
- * your program to something other than the user's locale, or if 
+ * calling <literal>setlocale (LC_ALL, "")</literal>. You would
+ * want to use this function if you wanted to set the locale for
+ * your program to something other than the user's locale, or if
  * you wanted to set different values for different locale categories.
  *
  * Most programs should not need to call this function.
@@ -625,6 +628,22 @@ setlocale_initialization (void)
     }
 }
 
+static void
+check_mixed_deps (void)
+{
+  GModule *module;
+  gpointer func;
+
+  module = g_module_open (NULL, 0);
+
+  if (g_module_symbol (module, "gtk_widget_device_is_shadowed", &func))
+    {
+      g_error ("GTK+ 3 symbols detected. Using GTK+ 2.x and GTK+ 3 in the same process is not supported");
+    }
+
+  g_module_close (module);
+}
+
 static void
 do_pre_parse_initialization (int    *argc,
 			     char ***argv)
@@ -643,6 +662,8 @@ do_pre_parse_initialization (int    *argc,
 
   pre_initialized = TRUE;
 
+  check_mixed_deps ();
+
   gdk_pre_parse_libgtk_only ();
   gdk_event_handler_set ((GdkEventFunc)gtk_main_do_event, NULL, NULL);
   
@@ -717,7 +738,6 @@ do_post_parse_initialization (int    *argc,
 
   /* do what the call to gtk_type_init() used to do */
   g_type_init ();
-  gtk_object_get_type ();
 
   _gtk_accel_map_init ();
   _gtk_rc_init ();
@@ -849,9 +869,9 @@ gtk_get_option_group (gboolean open_default_display)
 gboolean
 gtk_init_with_args (int            *argc,
 		    char         ***argv,
-		    char           *parameter_string,  
+		    const char     *parameter_string,
 		    GOptionEntry   *entries,
-		    char           *translation_domain,
+		    const char     *translation_domain,
 		    GError        **error)
 {
   GOptionContext *context;
@@ -883,9 +903,10 @@ gtk_init_with_args (int            *argc,
 
 /**
  * gtk_parse_args:
- * @argc: a pointer to the number of command line arguments.
- * @argv: a pointer to the array of command line arguments.
- * 
+ * @argc: (inout): a pointer to the number of command line arguments
+ * @argv: (array length=argc) (inout): a pointer to the array of
+ *     command line arguments
+ *
  * Parses command line arguments, and initializes global
  * attributes of GTK+, but does not actually open a connection
  * to a display. (See gdk_display_open(), gdk_get_display_arg_name())
@@ -893,7 +914,7 @@ gtk_init_with_args (int            *argc,
  * Any arguments used by GTK+ or GDK are removed from the array and
  * @argc and @argv are updated accordingly.
  *
- * You shouldn't call this function explicitely if you are using
+ * There is no need to call this function explicitely if you are using
  * gtk_init(), or gtk_init_check().
  *
  * Return value: %TRUE if initialization succeeded, otherwise %FALSE.
@@ -936,13 +957,13 @@ gtk_parse_args (int    *argc,
 
 /**
  * gtk_init_check:
- * @argc: Address of the <parameter>argc</parameter> parameter of your 
+ * @argc: (inout): Address of the <parameter>argc</parameter> parameter of your
  *   main() function. Changed if any arguments were handled.
- * @argv: Address of the <parameter>argv</parameter> parameter of main(). 
+ * @argv: (array length=argc) (inout) (allow-none): Address of the <parameter>argv</parameter> parameter of main().
  *   Any parameters understood by gtk_init() are stripped before return.
- * 
- * This function does the same work as gtk_init() with only 
- * a single change: It does not terminate the program if the GUI can't be 
+ *
+ * This function does the same work as gtk_init() with only
+ * a single change: It does not terminate the program if the GUI can't be
  * initialized. Instead it returns %FALSE on failure.
  *
  * This way the application can fall back to some other means of communication 
@@ -967,27 +988,30 @@ gtk_init_check (int	 *argc,
 
 /**
  * gtk_init:
- * @argc: Address of the <parameter>argc</parameter> parameter of your 
- *   main() function. Changed if any arguments were handled.
- * @argv: Address of the <parameter>argv</parameter> parameter of main(). 
- *   Any parameters understood by gtk_init() are stripped before return.
- * 
- * Call this function before using any other GTK+ functions in your GUI
- * applications.  It will initialize everything needed to operate the 
- * toolkit and parses some standard command line options. @argc and 
- * @argv are adjusted accordingly so your own code will 
- * never see those standard arguments. 
+ * @argc: (inout): Address of the <parameter>argc</parameter> parameter of
+ *     your main() function. Changed if any arguments were handled
+ * @argv: (array length=argc) (inout) (allow-none): Address of the
+ *     <parameter>argv</parameter> parameter of main(). Any options
+ *     understood by GTK+ are stripped before return.
  *
- * Note that there are some alternative ways to initialize GTK+: 
- * if you are calling gtk_parse_args(), gtk_init_check(), 
- * gtk_init_with_args() or g_option_context_parse() with 
- * the option group returned by gtk_get_option_group(), you 
- * <emphasis>don't</emphasis> have to call gtk_init().
+ * Call this function before using any other GTK+ functions in your GUI
+ * applications.  It will initialize everything needed to operate the
+ * toolkit and parses some standard command line options.
+ *
+ * @argc and @argv are adjusted accordingly so your own code will
+ * never see those standard arguments.
+ *
+ * Note that there are some alternative ways to initialize GTK+:
+ * if you are calling gtk_parse_args(), gtk_init_check(),
+ * gtk_init_with_args() or g_option_context_parse() with
+ * the option group returned by gtk_get_option_group(),
+ * you <emphasis>don't</emphasis> have to call gtk_init().
  *
  * <note><para>
- * This function will terminate your program if it was unable to initialize 
- * the GUI for some reason. If you want your program to fall back to a 
- * textual interface you want to call gtk_init_check() instead.
+ * This function will terminate your program if it was unable to
+ * initialize the windowing system for some reason. If you want
+ * your program to fall back to a textual interface you want to
+ * call gtk_init_check() instead.
  * </para></note>
  *
  * <note><para>
@@ -998,7 +1022,7 @@ gtk_init_check (int	 *argc,
  * but notice that other libraries (e.g. libdbus or gvfs) might do
  * similar things.
  * </para></note>
- **/
+ */
 void
 gtk_init (int *argc, char ***argv)
 {
@@ -1098,6 +1122,8 @@ gtk_exit (gint errorcode)
  * result of the setlocale(); it is also used on other machines, such as 
  * Windows, where the C library returns a different result. The string is 
  * owned by GTK+ and should not be modified or freed.
+ *
+ * Deprecated: 2.24: Use setlocale() directly
  **/
 gchar *
 gtk_set_locale (void)
@@ -1418,7 +1444,7 @@ rewrite_event_for_grabs (GdkEvent *event)
     case GDK_MOTION_NOTIFY:
     case GDK_PROXIMITY_IN:
     case GDK_PROXIMITY_OUT:
-      display = gdk_drawable_get_display (event->proximity.window);
+      display = gdk_window_get_display (event->proximity.window);
       if (!gdk_pointer_grab_info_libgtk_only (display, &grab_window, &owner_events) ||
 	  !owner_events)
 	return NULL;
@@ -1426,7 +1452,7 @@ rewrite_event_for_grabs (GdkEvent *event)
 
     case GDK_KEY_PRESS:
     case GDK_KEY_RELEASE:
-      display = gdk_drawable_get_display (event->key.window);
+      display = gdk_window_get_display (event->key.window);
       if (!gdk_keyboard_grab_info_libgtk_only (display, &grab_window, &owner_events) ||
 	  !owner_events)
 	return NULL;
@@ -1518,7 +1544,7 @@ gtk_main_do_event (GdkEvent *event)
        *  then we send the event to the original event widget.
        *  This is the key to implementing modality.
        */
-      if (GTK_WIDGET_IS_SENSITIVE (event_widget) &&
+      if (gtk_widget_is_sensitive (event_widget) &&
 	  gtk_widget_is_ancestor (event_widget, grab_widget))
 	grab_widget = event_widget;
     }
@@ -1558,14 +1584,14 @@ gtk_main_do_event (GdkEvent *event)
 	{
 	  g_object_ref (event_widget);
 	  if (!gtk_widget_event (event_widget, event) &&
-	      GTK_WIDGET_REALIZED (event_widget))
+	      gtk_widget_get_realized (event_widget))
 	    gtk_widget_destroy (event_widget);
 	  g_object_unref (event_widget);
 	}
       break;
       
     case GDK_EXPOSE:
-      if (event->any.window && GTK_WIDGET_DOUBLE_BUFFERED (event_widget))
+      if (event->any.window && gtk_widget_get_double_buffered (event_widget))
 	{
 	  gdk_window_begin_paint_region (event->any.window, event->expose.region);
 	  gtk_widget_send_expose (event_widget, event);
@@ -1614,6 +1640,30 @@ gtk_main_do_event (GdkEvent *event)
 	  if (gtk_invoke_key_snoopers (grab_widget, event))
 	    break;
 	}
+      /* Catch alt press to enable auto-mnemonics;
+       * menus are handled elsewhere
+       */
+      if ((event->key.keyval == GDK_Alt_L || event->key.keyval == GDK_Alt_R) &&
+          !GTK_IS_MENU_SHELL (grab_widget))
+        {
+          gboolean auto_mnemonics;
+
+          g_object_get (gtk_widget_get_settings (grab_widget),
+                        "gtk-auto-mnemonics", &auto_mnemonics, NULL);
+
+          if (auto_mnemonics)
+            {
+              gboolean mnemonics_visible;
+              GtkWidget *window;
+
+              mnemonics_visible = (event->type == GDK_KEY_PRESS);
+
+              window = gtk_widget_get_toplevel (grab_widget);
+
+              if (GTK_IS_WINDOW (window))
+                gtk_window_set_mnemonics_visible (GTK_WINDOW (window), mnemonics_visible);
+            }
+        }
       /* else fall through */
     case GDK_MOTION_NOTIFY:
     case GDK_BUTTON_RELEASE:
@@ -1625,13 +1675,13 @@ gtk_main_do_event (GdkEvent *event)
     case GDK_ENTER_NOTIFY:
       GTK_PRIVATE_SET_FLAG (event_widget, GTK_HAS_POINTER);
       _gtk_widget_set_pointer_window (event_widget, event->any.window);
-      if (GTK_WIDGET_IS_SENSITIVE (grab_widget))
+      if (gtk_widget_is_sensitive (grab_widget))
 	gtk_widget_event (grab_widget, event);
       break;
       
     case GDK_LEAVE_NOTIFY:
       GTK_PRIVATE_UNSET_FLAG (event_widget, GTK_HAS_POINTER);
-      if (GTK_WIDGET_IS_SENSITIVE (grab_widget))
+      if (gtk_widget_is_sensitive (grab_widget))
 	gtk_widget_event (grab_widget, event);
       break;
       
@@ -1734,7 +1784,7 @@ gtk_grab_notify_foreach (GtkWidget *child,
     {
       GTK_PRIVATE_SET_FLAG (child, GTK_SHADOWED);
       if (!was_shadowed && GTK_WIDGET_HAS_POINTER (child)
-	  && GTK_WIDGET_IS_SENSITIVE (child))
+	  && gtk_widget_is_sensitive (child))
 	_gtk_widget_synthesize_crossing (child, info->new_grab_widget,
 					 GDK_CROSSING_GTK_GRAB);
     }
@@ -1742,7 +1792,7 @@ gtk_grab_notify_foreach (GtkWidget *child,
     {
       GTK_PRIVATE_UNSET_FLAG (child, GTK_SHADOWED);
       if (was_shadowed && GTK_WIDGET_HAS_POINTER (child)
-	  && GTK_WIDGET_IS_SENSITIVE (child))
+	  && gtk_widget_is_sensitive (child))
 	_gtk_widget_synthesize_crossing (info->old_grab_widget, child,
 					 info->from_grab ? GDK_CROSSING_GTK_GRAB
 					 : GDK_CROSSING_GTK_UNGRAB);
@@ -1802,9 +1852,9 @@ gtk_grab_add (GtkWidget *widget)
   
   g_return_if_fail (widget != NULL);
   
-  if (!GTK_WIDGET_HAS_GRAB (widget) && GTK_WIDGET_IS_SENSITIVE (widget))
+  if (!gtk_widget_has_grab (widget) && gtk_widget_is_sensitive (widget))
     {
-      GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_GRAB);
+      _gtk_widget_set_has_grab (widget, TRUE);
       
       group = gtk_main_get_window_group (widget);
 
@@ -1820,6 +1870,14 @@ gtk_grab_add (GtkWidget *widget)
     }
 }
 
+/**
+ * gtk_grab_get_current:
+ *
+ * Queries the current grab of the default window group.
+ *
+ * Return value: (transfer none): The widget which currently
+ *     has the grab or %NULL if no grab is active
+ */
 GtkWidget*
 gtk_grab_get_current (void)
 {
@@ -1840,9 +1898,9 @@ gtk_grab_remove (GtkWidget *widget)
   
   g_return_if_fail (widget != NULL);
   
-  if (GTK_WIDGET_HAS_GRAB (widget))
+  if (gtk_widget_has_grab (widget))
     {
-      GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_GRAB);
+      _gtk_widget_set_has_grab (widget, FALSE);
 
       group = gtk_main_get_window_group (widget);
       group->grabs = g_slist_remove (group->grabs, widget);
@@ -2228,7 +2286,8 @@ gtk_invoke_input (gpointer	    data,
  * signal. The returned event must be freed with gdk_event_free().
  * If there is no current event, the function returns %NULL.
  * 
- * Return value: a copy of the current event, or %NULL if no current event.
+ * Return value: (transfer full): a copy of the current event, or %NULL if no
+ *     current event.
  **/
 GdkEvent*
 gtk_get_current_event (void)
@@ -2288,7 +2347,8 @@ gtk_get_current_event_state (GdkModifierType *state)
  * returns %NULL, otherwise returns the widget that received the event
  * originally.
  * 
- * Return value: the widget that originally received @event, or %NULL
+ * Return value: (transfer none): the widget that originally
+ *     received @event, or %NULL
  **/
 GtkWidget*
 gtk_get_event_widget (GdkEvent *event)
@@ -2379,7 +2439,7 @@ gtk_propagate_event (GtkWidget *widget,
 	  /* If there is a grab within the window, give the grab widget
 	   * a first crack at the key event
 	   */
-	  if (widget != window && GTK_WIDGET_HAS_GRAB (widget))
+	  if (widget != window && gtk_widget_has_grab (widget))
 	    handled_event = gtk_widget_event (widget, event);
 	  
 	  if (!handled_event)
@@ -2387,7 +2447,7 @@ gtk_propagate_event (GtkWidget *widget,
 	      window = gtk_widget_get_toplevel (widget);
 	      if (GTK_IS_WINDOW (window))
 		{
-		  if (GTK_WIDGET_IS_SENSITIVE (window))
+		  if (gtk_widget_is_sensitive (window))
 		    gtk_widget_event (window, event);
 		}
 	    }
@@ -2411,7 +2471,7 @@ gtk_propagate_event (GtkWidget *widget,
 	   * to have children of the viewport eat the scroll
 	   * event
 	   */
-	  if (!GTK_WIDGET_IS_SENSITIVE (widget))
+	  if (!gtk_widget_is_sensitive (widget))
 	    handled_event = event->type != GDK_SCROLL;
 	  else
 	    handled_event = gtk_widget_event (widget, event);
@@ -2530,7 +2590,7 @@ gtk_print (gchar *str)
 				 G_CALLBACK (gtk_widget_hide),
 				 GTK_OBJECT (window));
       gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
-      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+      gtk_widget_set_can_default (button, TRUE);
       gtk_widget_grab_default (button);
       gtk_widget_show (button);
     }
@@ -2539,7 +2599,7 @@ gtk_print (gchar *str)
   gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, str, -1);
   level -= 1;
   
-  if (!GTK_WIDGET_VISIBLE (window))
+  if (!gtk_widget_get_visible (window))
     gtk_widget_show (window);
 }
 #endif
